package com.hcloud.microserver.bank.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.bank.bo.CustomerInfoBO;
import com.hcloud.microserver.bank.dao.*;
import com.hcloud.microserver.bank.enums.LoginChannelEnum;
import com.hcloud.microserver.bank.service.CustomerCompanyInfoService;
import com.hcloud.microserver.bank.service.CustomerInfoService;
import com.hcloud.microserver.commoncore.base.PageBean;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.commoncore.util.GetUUID;
import com.hcloud.microserver.commoncore.util.Md5Util;
import com.hcloud.microserver.commoncore.util.string.StringUtils;
import com.hcloud.microserver.facade.carbon.entity.CustomerChannel;
import com.hcloud.microserver.facade.carbon.entity.CustomerInfo;
import com.hcloud.microserver.facade.carbon.forms.CustomerCompanyInfoForm;
import com.hcloud.microserver.facade.carbon.forms.CustomerInfoForm;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @author xiezt
 */
@Transactional(rollbackFor = Exception.class)
@Slf4j
@Service("customerInfoService")
public class CustomerInfoServiceImpl implements CustomerInfoService {


    @Resource
    private CustomerInfoMapper customerInfoMapper;

    @Resource
    private CustomerChannelMapper customerChannelMapper;

    @Autowired
    private CustomerCompanyInfoService customerCompanyInfoService;

    @Resource
    private GoodsOrderInfoMapper goodsOrderInfoMapper;

    @Resource
    private GoodsPayOrderMapper goodsPayOrderMapper;

    @Resource
    private OrderPayApplyMapper orderPayApplyMapper;

    @Resource
    private OrderPayVoucherMapper orderPayVoucherMapper;

    /**
     * 分页查询爱心用户信息
     *
     * @param customerInfoForm
     * @return
     */
    @Override
    public PageInfo searchCustomerByPage(CustomerInfoForm customerInfoForm) {
        PageBean pageBean = new PageBean(customerInfoForm);
        PageHelper.startPage(customerInfoForm.getPageNo(),customerInfoForm.getPageSize());
        List<CustomerInfoBO> list =  customerInfoMapper.searchCustomerByPage(pageBean);
        return new PageInfo(list);
    }

    /**
     * 通过Id查询爱心用户信息
     *
     * @param customerId
     * @return
     */
    @Override
    public CustomerInfoForm getCustomerInfoById(String customerId) {
        CustomerInfo customerInfo = customerInfoMapper.selectByPrimaryKey(customerId);
        CustomerInfoForm customerInfoForm = BeanCopyUtil.convertBean(customerInfo,CustomerInfoForm.class);
        log.info("customerInfoForm==========================>{}",customerInfoForm);
        //统计爱用用户业务信息
        CustomerInfoBO infoBO = customerInfoMapper.getCustomerStatisticalInfo(customerId);
        if(null!=infoBO){
            customerInfoForm.setCarbonAmount(infoBO.getCarbonAmount());
            customerInfoForm.setGoodsNum(infoBO.getGoodsNum());
            customerInfoForm.setFarmerNum(infoBO.getFarmerNum());
        }
        //企业认证信息
        CustomerCompanyInfoForm companyInfoForm = customerCompanyInfoService.getCustomerCompanyInfoByCustomerId(customerId);
        if(companyInfoForm!=null){
            customerInfoForm.setCustomerCompanyInfoForm(companyInfoForm);
        }
        return customerInfoForm;
    }

    /**
     * 通过账号查询爱心用户信息
     *
     * @param account
     * @return
     */
    @Override
    public CustomerInfoForm getCustomerInfoByAccount(String account) {
        CustomerInfo customerInfo = customerInfoMapper.searchCustomerByAccount(account);
        if(customerInfo==null){return null;}
        return BeanCopyUtil.convertBean(customerInfo,CustomerInfoForm.class);
    }

    /**
     * 通过多条件查询爱心用户信息
     *
     * @param queryStr
     * @return
     */
    @Override
    public CustomerInfoForm getCustomerInfoByInfo(String queryStr) {
        CustomerInfo customerInfo =  customerInfoMapper.getCustomerByInfo(queryStr);
        if(Objects.isNull(customerInfo)){return null;}
        return BeanCopyUtil.convertBean(customerInfo,CustomerInfoForm.class);
    }

    /**
     * 通过登录渠道查询用户信息
     *
     * @param channelType
     * @param threeCode
     * @return
     */
    @Override
    public CustomerInfoForm getCustomerInfoByChannelInfo(Integer channelType, String threeCode) {
        CustomerInfoBO customerInfo =customerInfoMapper.getCustomerByThreeCode(channelType,threeCode);
        log.info("===================>{}",customerInfo);
        if(customerInfo==null){return new CustomerInfoForm();}
        return BeanCopyUtil.convertBean(customerInfo,CustomerInfoForm.class);
    }

    /**
     * 通过登录渠道信息保存爱心用户
     *
     * @param customerInfoForm
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int saveCustomerInfoByChannelInfo(CustomerInfoForm customerInfoForm) {
        //保存用户信息
        int i = this.saveSelective(customerInfoForm);
        if(i==0){throw new RuntimeException();}
        //保存用户渠道信息
        Integer channelType = customerInfoForm.getLoginType();
        if(channelType!=null){
            CustomerChannel customerChannel = new CustomerChannel();
            customerChannel.setGuid(GetUUID.getUUID());
            customerChannel.setFkCustomerId(customerInfoForm.getGuid());
            customerChannel.setChannelKeywordOne(customerInfoForm.getUnionId());
            customerChannel.setChannelKeywordTwo(customerInfoForm.getWxId());
            customerChannel.setChannelName(LoginChannelEnum.getChannelNameByType(channelType));
            customerChannel.setChannelType(channelType);
            customerChannelMapper.insertSelective(customerChannel);
        }
        return i;
    }

    @Override
    public int resetPassword(CustomerInfoForm customerInfoForm) {
        if (Objects.nonNull(customerInfoForm)) {
            CustomerInfo customerInfo = BeanCopyUtil.convertBean(customerInfoForm, CustomerInfo.class);
            String guid = customerInfo.getGuid();
            if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
                customerInfo.setPasswd(Md5Util.toMD5("000000"));
                return this.customerInfoMapper.updateByPrimaryKeySelective(customerInfo);
            }
        }
        return 0;
    }

    /**
     * 更新用户信息（含微信登录渠道，及微信编号信息）
     *
     * @param customerInfoForm
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int updateCustomerInfoForWeChat(CustomerInfoForm customerInfoForm) {
        int i=this.modifyByPrimaryKeySelective(customerInfoForm);
        if(i>0){
           CustomerChannel customerChannel =
                   customerChannelMapper.getChannelInfoByCustomerId(customerInfoForm.getGuid(),customerInfoForm.getChannelType());
           if(Objects.isNull(customerChannel)){
               CustomerChannel innerChannel = new CustomerChannel();
               innerChannel.setGuid(GetUUID.getUUID());
               innerChannel.setFkCustomerId(customerInfoForm.getGuid());
               innerChannel.setChannelName("weChat");
               innerChannel.setChannelKeywordOne(customerInfoForm.getUnionId());
               innerChannel.setChannelKeywordTwo(customerInfoForm.getWxId());
               innerChannel.setChannelType(1);
               int h = customerChannelMapper.insertSelective(innerChannel);
               if(h==0){throw new RuntimeException();}
               return h;
           }else{
               customerChannel.setChannelKeywordOne(customerInfoForm.getUnionId()==null?"":customerInfoForm.getUnionId());
               customerChannel.setChannelKeywordTwo(customerInfoForm.getWxId()==null?"":customerInfoForm.getWxId());
               int j = customerChannelMapper.updateByPrimaryKeySelective(customerChannel);
               if(j==0){throw new RuntimeException();}
               return i;
           }
        }
        return 0;
    }

    /**
     * 更新用户订单信息，到新的用户名下
     *
     * @param oldCustomerId
     * @param newCustomerId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int changeCustomerOrderInfo(String oldCustomerId, String newCustomerId) {
        //TODO 更新订单信息
        int i = goodsOrderInfoMapper.updateOrderBelongInfo(oldCustomerId,newCustomerId);
        if(i==0){throw new RuntimeException();}
        //TODO 更新订单支付流水信息
        int j = goodsPayOrderMapper.updatePayOrderBelongInfo(oldCustomerId,newCustomerId);
        if(j==0){throw new RuntimeException();}
        //TODO 更新离线支付申请
        int h = orderPayApplyMapper.updateOrderPayApplyBelongInfo(oldCustomerId,newCustomerId);
        if(h==0){throw new RuntimeException();}
        //TODO 更新离线支付凭证信息
        int k = orderPayVoucherMapper.updatePayVoucherBelongInfo(oldCustomerId,newCustomerId);
        if(k==0){throw new RuntimeException();}
        return i;
    }

    /**
     * 绑定用户手机号
     *
     * @param customerId
     * @param phone
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int bindCustomerPhone(String customerId, String phone) {
        CustomerInfoForm customerInfo = new CustomerInfoForm();
        customerInfo.setGuid(customerId);
        customerInfo.setPhone(phone);
        int i = this.modifyByPrimaryKeySelective(customerInfo);
        if(i==0){throw new RuntimeException();}
        //查询手机号码是否已经注册为用户
        CustomerInfoForm customerInfoForm = this.getCustomerInfoByInfo(phone);
        if(Objects.nonNull(customerInfoForm) && !"".equals(customerInfoForm.getGuid())){
            //更新（订单信息、支付订单信息、离线支付申请，离线支付凭证）
            String oldCustomerId = customerInfoForm.getGuid();
            int j =this.changeCustomerOrderInfo(oldCustomerId,customerId);
            if(j==0){throw new RuntimeException();}
            //逻辑删除老用户信息
            CustomerInfoForm oldInfo = new CustomerInfoForm();
            oldInfo.setGuid(oldCustomerId);
            oldInfo.setState(0);
            oldInfo.setModifiedTime(new Date());
            int k =this.modifyByPrimaryKeySelective(oldInfo);
            if(k==0){throw new RuntimeException();}
        }
        return i;
    }


    @Override
    public int delByPrimaryKey(String guid) {
        return customerInfoMapper.deleteByPrimaryKey(guid);
    }

    @Override
    public int save(CustomerInfoForm record) {
        record.setGuid(GetUUID.getUUID());
        CustomerInfo customerInfo = BeanCopyUtil.convertBean(record,CustomerInfo.class);
        return customerInfoMapper.insert(customerInfo);
    }

    @Override
    public int saveSelective(CustomerInfoForm record) {
        record.setGuid(GetUUID.getUUID());
        CustomerInfo customerInfo = BeanCopyUtil.convertBean(record,CustomerInfo.class);
        return customerInfoMapper.insertSelective(customerInfo);
    }

    @Override
    public CustomerInfoForm queryByPrimaryKey(String guid) {
        CustomerInfo customerInfo = customerInfoMapper.selectByPrimaryKey(guid);
        return BeanCopyUtil.convertBean(customerInfo,CustomerInfoForm.class);
    }

    @Override
    public int modifyByPrimaryKeySelective(CustomerInfoForm record) {
        CustomerInfo customerInfo = BeanCopyUtil.convertBean(record,CustomerInfo.class);
        return customerInfoMapper.updateByPrimaryKeySelective(customerInfo);
    }

    @Override
    public int modifyByPrimaryKeyWithBLOBs(CustomerInfoForm record) {
        // empty implements
        return 0;
    }

    @Override
    public int modifyByPrimaryKey(CustomerInfoForm record) {
        CustomerInfo customerInfo = BeanCopyUtil.convertBean(record,CustomerInfo.class);
        return customerInfoMapper.updateByPrimaryKey(customerInfo);
    }


}
