package com.hcloud.microserver.bank.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.bank.bo.FarmerInfoBO;
import com.hcloud.microserver.bank.dao.CollectiveFarmerMapper;
import com.hcloud.microserver.bank.dao.FarmerInfoMapper;
import com.hcloud.microserver.bank.service.FarmerInfoService;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.commoncore.util.BusinessCodeRule;
import com.hcloud.microserver.commoncore.util.UUIDUtils;
import com.hcloud.microserver.facade.carbon.entity.CollectiveFarmer;
import com.hcloud.microserver.facade.carbon.entity.FarmerInfo;
import com.hcloud.microserver.facade.carbon.forms.FarmerInfoForm;
import com.hcloud.microserver.facade.carbon.forms.PovertyAreaForm;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @author JYJ
 */
@Slf4j
@Service
public class FarmerInfoServiceImpl implements FarmerInfoService {

    private static String CODE_PREFIX = "F01";
    private static final SimpleDateFormat YEAR_SDF = new SimpleDateFormat("YY");

    @Resource
    private FarmerInfoMapper farmerInfoMapper;

    @Resource
    private CollectiveFarmerMapper collectiveFarmerMapper;

    private Integer getCurrentCode() {
        StringBuffer sbb = new StringBuffer();
        String maxCode = this.farmerInfoMapper.getCurrentCode();
        if (maxCode == null) {
            return 0;
        }
        String yearStr = YEAR_SDF.format(new Date());
        sbb.append(CODE_PREFIX);
        sbb.append(yearStr);
        String currentNum = maxCode.substring(sbb.toString().length());
        return Integer.valueOf(currentNum);
    }

    private String getMaxCode() {
        Integer currentNum = getCurrentCode();
        String maxCode = BusinessCodeRule.firstSpecialCodeRule(CODE_PREFIX, currentNum, 20);
        return maxCode;
    }

    @Override
    public int save(FarmerInfoForm farmerInfoForm) {
        if (Objects.nonNull(farmerInfoForm)) {
            FarmerInfo farmerInfo = BeanCopyUtil.convertBean(farmerInfoForm, FarmerInfo.class);
            farmerInfo.setGuid(UUIDUtils.randomUUID());
            farmerInfo.setFarmerCode(this.getMaxCode());
            farmerInfo.setCreateTime(new Date());
            farmerInfo.setModifiedTime(new Date());
            farmerInfo.setState(1);
            farmerInfo.setIsEnable(0);
            return this.farmerInfoMapper.insertSelective(farmerInfo);
        }
        return 0;
    }

    @Override
    public int softRemove(FarmerInfoForm farmerInfoForm) {
        if (Objects.nonNull(farmerInfoForm)) {
            String guid = farmerInfoForm.getGuid();
            if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
                FarmerInfo farmerInfo = BeanCopyUtil.convertBean(farmerInfoForm, FarmerInfo.class);
                farmerInfo.setState(0);
                farmerInfo.setModifiedTime(new Date());
                return this.farmerInfoMapper.updateByPrimaryKey(farmerInfo);
            }
        }
        return 0;
    }

    @Override
    public int modify(FarmerInfoForm farmerInfoForm) {
        if (Objects.nonNull(farmerInfoForm)) {
            String guid = farmerInfoForm.getGuid();
            if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
                FarmerInfo farmerInfo = BeanCopyUtil.convertBean(farmerInfoForm, FarmerInfo.class);
                farmerInfo.setModifiedTime(new Date());
                return this.farmerInfoMapper.updateByPrimaryKey(farmerInfo);
            }
        }
        return 0;
    }

    @Override
    public PageInfo queryListByPage(FarmerInfoForm farmerInfoForm) {
        PageHelper.startPage(farmerInfoForm.getPageNo(), farmerInfoForm.getPageSize());
        Page<Map> page = this.farmerInfoMapper.selectListByPage(farmerInfoForm);
        if (Objects.nonNull(page) && page.size() > 0) {
            return new PageInfo(page);
        }
        return null;
    }

    @Override
    public int verify(FarmerInfoForm farmerInfoForm) {
        // TODO 审核卡
        return 0;
    }

    /**
     * 批量更新贫困户
     *
     * @author 朱修贤
     * @param farmerInfoForms
     * @return
     */
    @Override
    @Transactional
    public int batchModify(List<FarmerInfoForm> farmerInfoForms) {
        if (Objects.nonNull(farmerInfoForms) && farmerInfoForms.size() > 0) {
            List<FarmerInfo> farmerInfos = new ArrayList<>();
            for (FarmerInfoForm farmerInfoForm : farmerInfoForms) {
                String guid = farmerInfoForm.getGuid();
                if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
                    FarmerInfo farmerInfo = BeanCopyUtil.convertBean(farmerInfoForm, FarmerInfo.class);
                    farmerInfo.setModifiedTime(new Date());
                    farmerInfos.add(farmerInfo);
                } else {
                    return 0;
                }
            }
            return this.farmerInfoMapper.batchUpdate(farmerInfos);
        }
        return 0;
    }

    @Override
    public List<FarmerInfo> queryByCollectiveId(String collectiveId) {
        if (StringUtils.isNotEmpty(collectiveId) && collectiveId.matches("^\\w{32}$"))
            return this.farmerInfoMapper.selectByCollectiveId(collectiveId);
        return null;
    }

    @Override
    public List<FarmerInfo> queryByProductGuid(String productGuid) {
        return this.farmerInfoMapper.selectByProductGuid(productGuid);
    }

    @Override
    public PageInfo searchFarmerInfoByPovertyArea(FarmerInfoForm farmerInfoForm) {
        String povertyAreaId = farmerInfoForm.getFkPovertyAreaId();
        if(null!=povertyAreaId && !"".equals(povertyAreaId)){
            PageHelper.startPage(farmerInfoForm.getPageNo(),farmerInfoForm.getPageSize());
            List<FarmerInfo> farmerInfoList =farmerInfoMapper.getFarmerInfoByPovertyArea(povertyAreaId);
            return new PageInfo(farmerInfoList);
        }
        return null;
    }

    /**
     * 查询扶贫区域内农户统计信息
     *
     * @param povertyAreaId
     * @return
     */
    @Override
    public PovertyAreaForm getPovertyAreaAnalysis(String povertyAreaId) {
        return null;
    }

    /**
     * 通过商品id及销售金额更新农户销售信息
     *
     * @param saleAmount
     * @param goodsId
     * @return
     */
    @Override
    public int updateFarmerSaleInfoByGoodsId(BigDecimal saleAmount, String goodsId) {
        //查询商品归属农户
        List<FarmerInfoBO> list = farmerInfoMapper.getFarmerInfoByGoods(goodsId);
        log.info("List<FarmerInfoBO>====================>{}",list);
        if(null!=list && list.size()>0){
            for (FarmerInfoBO bo:list) {
                log.info("farmerBo====================>{}",bo);
                if(Objects.isNull(bo)){continue;}
                String farmerId = bo.getGuid();
                String collectiveId = bo.getCollectiveId();
                //判断购碳资金应打入什么账号（个人账户，集体账号）
                if(null!=collectiveId && !"".equals(collectiveId)){
                    //更新集体贫困账户销售信息
                    CollectiveFarmer collectiveFarmer = new CollectiveFarmer();
                    collectiveFarmer.setGuid(collectiveId);
                    collectiveFarmer.setRecieveMoney(bo.getRecieveMoney());
                    collectiveFarmer.setActualMoney(bo.getActualMoney());
                    collectiveFarmer.setModifiedTime(new Date());
                    collectiveFarmerMapper.updateByPrimaryKeySelective(collectiveFarmer);
                }else{
                    FarmerInfo farmerInfo = new FarmerInfo();
                    farmerInfo.setGuid(farmerId);
                    farmerInfo.setRecieveMoney(bo.getRecieveMoney());
                    farmerInfo.setActualMoney(bo.getActualMoney());
                    farmerInfo.setModifiedTime(new Date());
                    farmerInfoMapper.updateByPrimaryKeySelective(farmerInfo);
                }
            }
            return 1;
        }
        return 0;
    }


    @Override
    public int delByPrimaryKey(String guid) {
        if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
            return this.farmerInfoMapper.deleteByPrimaryKey(guid);
        }
        return 0;
    }

    @Override
    public int save(FarmerInfo record) {
        record.setGuid(UUIDUtils.randomUUID());
        record.setCreateTime(new Date());
        record.setModifiedTime(new Date());
        record.setIsEnable(1);
        record.setState(1);
        return this.farmerInfoMapper.insert(record);
    }

    @Override
    public int saveSelective(FarmerInfo record) {
        record.setGuid(UUIDUtils.randomUUID());
        record.setCreateTime(new Date());
        record.setModifiedTime(new Date());
        record.setIsEnable(1);
        record.setState(1);
        return this.farmerInfoMapper.insertSelective(record);
    }

    @Override
    public FarmerInfo queryByPrimaryKey(String guid) {
        if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
            return this.farmerInfoMapper.selectByPrimaryKey(guid);
        }
        return null;
    }

    @Override
    public int modifyByPrimaryKeySelective(FarmerInfo record) {
        if (Objects.nonNull(record)) {
            String guid = record.getGuid();
            if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
                record.setModifiedTime(new Date());
                return this.farmerInfoMapper.updateByPrimaryKeySelective(record);
            }
        }
        return 0;
    }

    @Override
    public int modifyByPrimaryKeyWithBLOBs(FarmerInfo record) {
        if (Objects.nonNull(record)) {
            String guid = record.getGuid();
            if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
                record.setModifiedTime(new Date());
                return this.farmerInfoMapper.updateByPrimaryKeyWithBLOBs(record);
            }
        }
        return 0;
    }

    @Override
    public int modifyByPrimaryKey(FarmerInfo record) {
        if (Objects.nonNull(record)) {
            String guid = record.getGuid();
            if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
                record.setModifiedTime(new Date());
                return this.farmerInfoMapper.updateByPrimaryKey(record);
            }
        }
        return 0;
    }
}
