package com.hcloud.microserver.bank.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.bank.dao.*;
import com.hcloud.microserver.bank.service.ProductInfoService;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.commoncore.util.BusinessCodeRule;
import com.hcloud.microserver.commoncore.util.GeneratorIdUtils;
import com.hcloud.microserver.commoncore.util.UUIDUtils;
import com.hcloud.microserver.facade.carbon.entity.*;
import com.hcloud.microserver.facade.carbon.forms.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

@Service
public class ProductInfoServiceImpl implements ProductInfoService {

    private static String CODE_PREFIX = "P01";
    private static String G_CODE_PREFIX = "G01";
    private static final SimpleDateFormat YEAR_SDF = new SimpleDateFormat("YY");

    @Resource
    private ProductInfoMapper productInfoMapper;

    @Resource
    private ProductBelongMapper productBelongMapper;

    @Resource
    private ProductExpansionMapper productExpansionMapper;

    @Resource
    private GoodsInfoMapper goodsInfoMapper;

    @Resource
    private GoodsDetailMapper goodsDetailMapper;

    @Resource
    private GoodsSaleInfoMapper goodsSaleInfoMapper;

    @Resource
    private CollectiveFarmerMapper collectiveFarmerMapper;

    @Resource
    private FarmerInfoMapper farmerInfoMapper;

    private static final GeneratorIdUtils ID_UTILS_INSTANCE = new GeneratorIdUtils();

    @Override
    public int delByPrimaryKey(String guid) {
        if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
            return this.productInfoMapper.deleteByPrimaryKey(guid);
        }
        return 0;
    }

    @Override
    public int save(ProductInfo record) {
        if (Objects.nonNull(record)) {
            record.setGuid(UUIDUtils.randomUUID());
            record.setCreateTime(new Date());
            record.setModifiedTime(new Date());
            record.setIsEnable(1);
            record.setState(1);
            return this.productInfoMapper.insert(record);
        }
        return 0;
    }

    @Override
    public int saveSelective(ProductInfo record) {
        if (Objects.nonNull(record)) {
            record.setGuid(UUIDUtils.randomUUID());
            record.setCreateTime(new Date());
            record.setModifiedTime(new Date());
            record.setIsEnable(1);
            record.setState(1);
            return this.productInfoMapper.insertSelective(record);
        }
        return 0;
    }

    @Override
    public ProductInfo queryByPrimaryKey(String guid) {
        if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
            return this.productInfoMapper.selectByPrimaryKey(guid);
        }
        return null;
    }

    @Override
    public int modifyByPrimaryKeySelective(ProductInfo record) {
        if (Objects.nonNull(record)) {
            String guid = record.getGuid();
            if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
                return this.modifyByPrimaryKeySelective(record);
            }
        }
        return 0;
    }

    @Override
    public int modifyByPrimaryKeyWithBLOBs(ProductInfo record) {
        if (Objects.nonNull(record)) {
            String guid = record.getGuid();
            if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
                return this.modifyByPrimaryKeyWithBLOBs(record);
            }
        }
        return 0;
    }

    @Override
    public int modifyByPrimaryKey(ProductInfo record) {
        if (Objects.nonNull(record)) {
            String guid = record.getGuid();
            if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
                return this.modifyByPrimaryKey(record);
            }
        }
        return 0;
    }

    /**
     * 新增产品信息
     *
     * @param productInfoForm 前端参数
     * @return
     * @author 朱修贤
     */
    @Override
    @Transactional
    public int save(ProductInfoForm productInfoForm) {
        int result = 0;
        if (Objects.nonNull(productInfoForm)) {
            ProductInfo productInfo = BeanCopyUtil.convertBean(productInfoForm, ProductInfo.class);
            // 产品信息必要字段赋值
            String guid = UUIDUtils.randomUUID();
            productInfo.setGuid(guid);
            productInfo.setCreateTime(new Date());
            productInfo.setModifiedTime(new Date());
            productInfo.setState(1);
            productInfo.setIsEnable(1);
            productInfo.setProductCode(getMaxProductCode());

            // 批量添加产品归属信息
            List<ProductBelongForm> productBelongForms = productInfoForm.getProductBelongForms();
            if (Objects.nonNull(productBelongForms) && productBelongForms.size() > 0) {
                List<ProductBelong> productBelongs = processBelong(guid, productBelongForms);
                result = this.productBelongMapper.batchInsert(productBelongs);
                if (result < 1) throw new RuntimeException("新增产品信息失败 -- 批量新增产品归属失败");
            }

            // 批量添加产品扩展信息
            List<ProductExpansionForm> productExpansionForms = productInfoForm.getProductExpansionForms();
            if (Objects.nonNull(productExpansionForms) && productExpansionForms.size() > 0) {
                List<ProductExpansion> productExpansions = new ArrayList<>();
                for (ProductExpansionForm productExpansionForm : productExpansionForms) {
                    ProductExpansion productExpansion = BeanCopyUtil.convertBean(productExpansionForm, ProductExpansion.class);
                    productExpansion.setCreateTime(new Date());
                    productExpansion.setFkProductGuid(guid);
                    productExpansion.setFkProductTypeGuid(productInfo.getFkProductType());
                    productExpansion.setModifiedTime(new Date());
                    productExpansion.setState(1);
                    productExpansion.setGuid(UUIDUtils.randomUUID());
                    productExpansions.add(productExpansion);
                }
                result = this.productExpansionMapper.batchInsert(productExpansions);
                if (result < 1) throw new RuntimeException("新增产品信息失败 -- 批量新增产品扩展信息失败");
            }

            // 如果前端选择集体户
            CollectiveFarmerForm collectiveFarmerForm = productInfoForm.getCollectiveFarmerForm();
            if (Objects.nonNull(collectiveFarmerForm)) {
                String collectiveGuid = collectiveFarmerForm.getGuid();
                if (StringUtils.isNotEmpty(collectiveGuid) && collectiveGuid.matches("^\\w{32}$")) {
                    CollectiveFarmer collectiveFarmer = this.collectiveFarmerMapper.selectByPrimaryKey(collectiveGuid);
                    List<FarmerInfo> farmerInfos = this.farmerInfoMapper.selectByCollectiveId(collectiveGuid);
                    if (Objects.nonNull(collectiveFarmer) && Objects.nonNull(farmerInfos)) {
                        List<ProductBelong> productBelongs = new ArrayList<>();
                        farmerInfos.forEach(farmerInfo -> {
                            ProductBelong productBelong = new ProductBelong();
                            productBelong.setGuid(UUIDUtils.randomUUID());
                            productBelong.setBelongType(1);
                            productBelong.setCreateTime(new Date());
                            productBelong.setFkProductGuid(guid);
                            productBelong.setFarmerName(farmerInfo.getFarmerName());
                            productBelong.setFarmerNum(farmerInfo.getFamilyNum());
                            productBelong.setFarmerVillage(collectiveFarmer.getCollectiveName());
                            productBelong.setFkFarmerGuid(farmerInfo.getGuid());
                            productBelong.setModifiedTime(new Date());
                            productBelong.setState(1);
                            productBelongs.add(productBelong);
                        });
                        if (Objects.nonNull(productBelongs) && productBelongs.size() > 0) {
                            result = this.productBelongMapper.batchInsert(productBelongs);
                            if (result < 1) {
                                throw new RuntimeException("新增产品信息 -- 批量新增产品归属失败");
                            }
                        }
                    }
                }
            }

            // 添加产品信息
            result = this.productInfoMapper.insert(productInfo);
            if (result < 1) throw new RuntimeException("新增产品信息失败");
        }
        return result;
    }

    /**
     * 删除产品信息(软删除)
     *
     * @param productInfoForm
     * @return
     * @author 朱修贤
     */
    @Override
    @Transactional
    public int remove(ProductInfoForm productInfoForm) {
        int result = 0;
        if (Objects.nonNull(productInfoForm)) {
            String guid = productInfoForm.getGuid();
            if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
                // 删除产品归属信息
                result = this.productBelongMapper.deleteByProductInfoGuid(guid);
                if (result < 0) throw new RuntimeException("删除产品归属信息失败");
                // 删除产品扩展信息
                result = this.productExpansionMapper.deleteByProductGuid(guid);
                if (result < 0) throw new RuntimeException("删除产品扩展信息失败");
                // 删除产品信息(软删除)
                result = this.productInfoMapper.softDelete(guid);
                if (result < 0) throw new RuntimeException("删除产品失败");
            }
        }
        return result;
    }

    /**
     * 修改产品信息
     *
     * @param productInfoForm
     * @return
     * @author 朱修贤
     */
    @Override
    @Transactional
    public int modify(ProductInfoForm productInfoForm) {
        int result = 0;
        if (Objects.nonNull(productInfoForm)) {
            String guid = productInfoForm.getGuid();
            if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
                ProductInfo productInfo = BeanCopyUtil.convertBean(productInfoForm, ProductInfo.class);
                // 删除数据库中对应的所有产品归属信息
                result = this.productBelongMapper.deleteByProductInfoGuid(guid);
                if (result < 0) throw new RuntimeException("修改产品信息失败 -- 删除产品归属信息失败");
                // 添加前端传递过来的归属信息
                List<ProductBelongForm> productBelongForms = productInfoForm.getProductBelongForms();
                if (Objects.nonNull(productBelongForms) && productBelongForms.size() > 0) {
                    List<ProductBelong> productBelongs = processBelong(guid, productBelongForms);
                    result = this.productBelongMapper.batchInsert(productBelongs);
                    if (result < 1) throw new RuntimeException("修改产品信息失败 -- 批量新增产品归属信息失败");
                }
                // 删除数据库中对应的所有产品扩展信息
                result = this.productExpansionMapper.deleteByProductGuid(guid);
                if (result < 0) throw new RuntimeException("修改产品信息失败 -- 删除产品扩展信息失败");
                // 添加前端传递过来的扩展信息
                List<ProductExpansionForm> productExpansionForms = productInfoForm.getProductExpansionForms();
                if (Objects.nonNull(productExpansionForms) && productExpansionForms.size() > 0) {
                    List<ProductExpansion> productExpansions = new ArrayList<>();
                    for (ProductExpansionForm productExpansionForm : productExpansionForms) {
                        ProductExpansion productExpansion = BeanCopyUtil.convertBean(productExpansionForm, ProductExpansion.class);
                        productExpansion.setCreateTime(new Date());
                        productExpansion.setFkProductGuid(guid);
                        productExpansion.setFkProductTypeGuid(productInfo.getFkProductType());
                        productExpansion.setModifiedTime(new Date());
                        productExpansion.setState(1);
                        productExpansion.setGuid(UUIDUtils.randomUUID());
                        productExpansions.add(productExpansion);
                    }
                    result = this.productExpansionMapper.batchInsert(productExpansions);
                    if (result < 0) throw new RuntimeException("修改产品信息失败 -- 批量新增产品扩展信息失败");
                }

                // 如果前端选择集体户
                CollectiveFarmerForm collectiveFarmerForm = productInfoForm.getCollectiveFarmerForm();
                if (Objects.nonNull(collectiveFarmerForm)) {
                    String collectiveGuid = collectiveFarmerForm.getGuid();
                    if (StringUtils.isNotEmpty(collectiveGuid) && collectiveGuid.matches("^\\w{32}$")) {
                        CollectiveFarmer collectiveFarmer = this.collectiveFarmerMapper.selectByPrimaryKey(collectiveGuid);
                        List<FarmerInfo> farmerInfos = this.farmerInfoMapper.selectByCollectiveId(collectiveGuid);
                        if (Objects.nonNull(collectiveFarmer) && Objects.nonNull(farmerInfos)) {
                            List<ProductBelong> productBelongs = new ArrayList<>();
                            farmerInfos.forEach(farmerInfo -> {
                                ProductBelong productBelong = new ProductBelong();
                                productBelong.setGuid(UUIDUtils.randomUUID());
                                productBelong.setBelongType(1);
                                productBelong.setCreateTime(new Date());
                                productBelong.setFkProductGuid(guid);
                                productBelong.setFarmerName(farmerInfo.getFarmerName());
                                productBelong.setFarmerNum(farmerInfo.getFamilyNum());
                                productBelong.setFarmerVillage(collectiveFarmer.getCollectiveName());
                                productBelong.setFkFarmerGuid(farmerInfo.getGuid());
                                productBelong.setModifiedTime(new Date());
                                productBelong.setState(1);
                                productBelongs.add(productBelong);
                            });
                            if (Objects.nonNull(productBelongs) && productBelongs.size() > 0) {
                                result = this.productBelongMapper.batchInsert(productBelongs);
                                if (result < 1) throw new RuntimeException("修改产品 -- 批量添加产品归属失败");
                            }
                        }
                    }
                }

                List<GoodsInfo> goodsInfos = this.goodsInfoMapper.selectGoodsInfoByProductGuid(guid);
                if (Objects.nonNull(goodsInfos) && goodsInfos.size() > 0) {
                    for (GoodsInfo goodsInfo : goodsInfos) {
                        goodsInfo.setGoodsImages(productInfo.getProductImgs());
                    }
                    result = this.goodsInfoMapper.batchUpdate(goodsInfos);
                    if (result < 0) throw new RuntimeException("修改产品 -- 批量更新商品失败");
                }

                result = this.productInfoMapper.updateByPrimaryKeySelective(productInfo);
                if (result < 1) throw new RuntimeException("修改产品信息失败");
            }
        }
        return result;
    }

    /**
     * 辅助方法(消除重复代码)
     *
     * @param guid
     * @param productBelongForms
     * @return
     */
    private List<ProductBelong> processBelong(String guid, List<ProductBelongForm> productBelongForms) {
        List<ProductBelong> productBelongs = new ArrayList<>();
        for (ProductBelongForm productBelongForm : productBelongForms) {
            ProductBelong productBelong = BeanCopyUtil.convertBean(productBelongForm, ProductBelong.class);
            productBelong.setGuid(UUIDUtils.randomUUID());
            productBelong.setFkProductGuid(guid);
            productBelong.setCreateTime(new Date());
            productBelong.setModifiedTime(new Date());
            productBelong.setState(1);
            productBelongs.add(productBelong);
        }
        return productBelongs;
    }

    /**
     * 产品列表 分页查询
     *
     * @param productInfoForm
     * @return
     * @author 朱修贤
     */
    @Override
    public PageInfo queryListByPage(ProductInfoForm productInfoForm) {
        PageHelper.startPage(productInfoForm.getPageNo(), productInfoForm.getPageSize());
        Page page = this.productInfoMapper.selectListByPage(productInfoForm);
        if (Objects.nonNull(page) && page.size() > 0) {
            return new PageInfo(page);
        }
        return null;
    }

    /**
     * 发布商品 生成商品
     *
     * @param productInfoForm
     * @param goodsInfoForm
     * @return
     * @author 朱修贤
     */
    @Override
    @Transactional
    public int publish(ProductInfoForm productInfoForm, GoodsInfoForm goodsInfoForm) {
        int result = 0;
        if (Objects.nonNull(productInfoForm) && Objects.nonNull(goodsInfoForm)) {
            ProductInfo productInfo = BeanCopyUtil.convertBean(productInfoForm, ProductInfo.class);
            String guid = productInfo.getGuid();
            if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
                GoodsInfo goodsInfo = BeanCopyUtil.convertBean(goodsInfoForm, GoodsInfo.class);
                String goodsGuid = UUIDUtils.randomUUID();
                goodsInfo.setGuid(goodsGuid);
                goodsInfo.setGoodsName(productInfo.getProductName());
                goodsInfo.setGoodsImages(productInfo.getProductImgs());
                goodsInfo.setGoodsDesc(productInfo.getRemark());
                goodsInfo.setGoodsCode(getMaxGoodsCode());
                goodsInfo.setCarbonVal(productInfo.getCarbonSink());
                goodsInfo.setGoodsType(productInfo.getProductType().getTypeVal());
                goodsInfo.setState(1);
                goodsInfo.setIsSale(1);
                goodsInfo.setCreateTime(new Date());
                goodsInfo.setModifiedTime(new Date());
                result = this.goodsInfoMapper.insert(goodsInfo);
                if (result < 1) throw new RuntimeException("产品发布 -- 添加商品记录失败");

                GoodsDetail goodsDetail = new GoodsDetail();
                goodsDetail.setGuid(UUIDUtils.randomUUID());
                goodsDetail.setCarbonVal(productInfo.getCarbonSink());
                goodsDetail.setCreateTime(new Date());
                goodsDetail.setModifiedTime(new Date());
                goodsDetail.setFkGoodsGuid(goodsGuid);
                goodsDetail.setFkProductGuid(guid);
                goodsDetail.setProductCode(productInfo.getProductCode());
                goodsDetail.setProductName(productInfo.getProductName());
                goodsDetail.setIsEnable(1);
                goodsDetail.setState(1);
                goodsDetail.setSaleUnit("公斤");
                result = this.goodsDetailMapper.insert(goodsDetail);
                if (result < 1) throw new RuntimeException("产品发布 -- 添加商品明细记录失败");

                GoodsSaleInfo goodsSaleInfo = new GoodsSaleInfo();
                goodsSaleInfo.setFkGoodsGuid(goodsGuid);
                goodsSaleInfo.setSalePrice(goodsInfoForm.getPrice());
                goodsSaleInfo.setGuid(UUIDUtils.randomUUID());
                goodsSaleInfo.setIsEnable(1);
                goodsSaleInfo.setState(1);
                goodsSaleInfo.setUnsaledCarbonSkin(goodsInfoForm.getCarbonVal());
                goodsSaleInfo.setSaledCarbonSkin(new BigDecimal(0));
                goodsSaleInfo.setSaleType(goodsInfoForm.getSaleType());
                goodsSaleInfo.setSaleTargets(goodsInfoForm.getSaleTargets());
                goodsSaleInfo.setModifiedTime(new Date());
                result = this.goodsSaleInfoMapper.insertSelective(goodsSaleInfo);
                if (result < 1) throw new RuntimeException("产品发布 -- 添加商品销售信息失败");

                productInfo.setIsEnable(2);
                result = productInfoMapper.updateByPrimaryKey(productInfo);
                if (result < 1) throw new RuntimeException("产品发布 -- 修改产品is_enable失败");
            }
        }
        return result;
    }

    @Override
    public int publishGoods(GoodsInfo goodsInfo, GoodsSaleInfo goodsSaleInfo, List<ProductInfo> productInfos) {
        int result = 0;
        boolean cond1 = Objects.nonNull(goodsInfo);
        boolean cond2 = Objects.nonNull(goodsSaleInfo);
        boolean cond3 = Objects.nonNull(productInfos) && productInfos.size() > 0;

        if (cond1 && cond2 && cond3) {
            // 生成商品记录的guid
            String goodsInfoGuid = UUIDUtils.randomUUID();
            goodsInfo.setGuid(goodsInfoGuid);
            goodsInfo.setState(1);
            goodsInfo.setModifiedTime(new Date());
            goodsInfo.setCreateTime(new Date());
            // 扶贫产品
            goodsInfo.setGoodsType(productInfos.get(0).getProductType().getTypeVal());
            goodsInfo.setGoodsCode(getMaxGoodsCode());
            goodsInfo.setIsSale(0);

            // 合计碳汇量
            BigDecimal totalCarbonSink = new BigDecimal(0);
            StringBuffer goodsImages = new StringBuffer();
            StringBuffer goodsName = new StringBuffer();

            // 生成商品明细
            List<GoodsDetail> goodsDetails = new ArrayList<>();
            productInfos.forEach(productInfo -> {
                totalCarbonSink.add(productInfo.getCarbonSink());
                goodsImages.append(productInfo.getProductImgs() + ";");
                goodsName.append(productInfo.getProductName() + ",");
                GoodsDetail goodsDetail = new GoodsDetail();
                goodsDetail.setGuid(UUIDUtils.randomUUID());
                goodsDetail.setFkGoodsGuid(goodsInfoGuid);
                goodsDetail.setCreateTime(new Date());
                goodsDetail.setModifiedTime(new Date());
                goodsDetail.setState(1);
                goodsDetail.setIsEnable(1);
                goodsDetail.setCarbonVal(productInfo.getCarbonSink());
                goodsDetail.setFkProductGuid(productInfo.getGuid());
                goodsDetail.setProductName(productInfo.getProductName());
                goodsDetail.setFarmerNums(productInfo.getProductBelongs().size());
                goodsDetail.setSaleUnit("公斤");
                goodsDetail.setProductCode(productInfo.getProductCode());
                goodsDetails.add(goodsDetail);
            });

            // 设置商品图片
            goodsInfo.setGoodsImages(goodsImages.substring(0, goodsImages.length() - 1));
            goodsInfo.setGoodsName(goodsName.substring(0, goodsName.length() - 1));

            // 商品销售信息表
            goodsSaleInfo.setGuid(UUIDUtils.randomUUID());
            goodsSaleInfo.setFkGoodsGuid(goodsInfoGuid);
            goodsSaleInfo.setSalePrice(goodsInfo.getPrice());
            goodsSaleInfo.setIsEnable(1);
            goodsSaleInfo.setState(1);
            goodsSaleInfo.setUnsaledCarbonSkin(totalCarbonSink);
            goodsSaleInfo.setSaledCarbonSkin(new BigDecimal(0));
            goodsSaleInfo.setModifiedTime(new Date());

            result = this.goodsDetailMapper.batchInsert(goodsDetails);
            if (result < 1){ throw new RuntimeException("添加商品明细失败");}
            result = this.goodsSaleInfoMapper.insertSelective(goodsSaleInfo);
            if (result < 1) {throw new RuntimeException("添加商品销售信息失败");}
            result = this.goodsInfoMapper.insertSelective(goodsInfo);
            if (result < 1) {throw new RuntimeException("添加商品信息失败");}

        }

        return result;
    }

    @Override
    public List<ProductInfo> searchByGoodsInfoGuid(String goodsInfoGuid) {
        if (StringUtils.isNotEmpty(goodsInfoGuid) && goodsInfoGuid.matches("^\\w{32}$")) {
            return this.productInfoMapper.selectByGoodsInfoGuid(goodsInfoGuid);
        }
        return null;
    }

    private Integer getCurrentProductCode() {
        StringBuffer sbb = new StringBuffer();
        String maxCode = this.productInfoMapper.getCurrentCode();
        if (maxCode == null) {
            return 0;
        }
        String yearStr = YEAR_SDF.format(new Date());
        sbb.append(CODE_PREFIX);
        sbb.append(yearStr);
        String currentNum = maxCode.substring(sbb.toString().length());
        return Integer.valueOf(currentNum);
    }

    private String getMaxProductCode() {
        Integer currentNum = getCurrentProductCode();
        String maxCode = BusinessCodeRule.firstSpecialCodeRule(CODE_PREFIX, currentNum, 15);
        return maxCode;
    }

    private Integer getCurrentGoodsCode() {
        StringBuffer sbb = new StringBuffer();
        String maxCode = this.goodsInfoMapper.getCurrentCode();
        if (maxCode == null) {
            return 0;
        }
        String yearStr = YEAR_SDF.format(new Date());
        sbb.append(G_CODE_PREFIX);
        sbb.append(yearStr);
        String currentNum = maxCode.substring(sbb.toString().length());
        return Integer.valueOf(currentNum);
    }

    private String getMaxGoodsCode() {
        Integer currentNum = getCurrentGoodsCode();
        String maxCode = BusinessCodeRule.firstSpecialCodeRule(G_CODE_PREFIX, currentNum, 20);
        return maxCode;
    }

}
