package com.hcloud.microserver.bank.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.bank.dao.GoodsInfoMapper;
import com.hcloud.microserver.bank.dao.ProductInfoMapper;
import com.hcloud.microserver.bank.dao.ProductTypeExpansionMapper;
import com.hcloud.microserver.bank.dao.ProductTypeMapper;
import com.hcloud.microserver.bank.service.ProductTypeService;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.commoncore.util.BusinessCodeRule;
import com.hcloud.microserver.commoncore.util.UUIDUtils;
import com.hcloud.microserver.commoncore.util.string.StringUtils;
import com.hcloud.microserver.facade.carbon.entity.ProductType;
import com.hcloud.microserver.facade.carbon.entity.ProductTypeExpansion;
import com.hcloud.microserver.facade.carbon.forms.ProductTypeExpansionForm;
import com.hcloud.microserver.facade.carbon.forms.ProductTypeForm;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author JYJ
 */
@Slf4j
@Service
public class ProductTypeServiceImpl implements ProductTypeService {

    private static final String CODE_PREFIX = "T";

    @Resource
    private ProductTypeMapper productTypeMapper;

    @Resource
    private ProductTypeExpansionMapper productTypeExpansionMapper;

    @Resource
    private GoodsInfoMapper goodsInfoMapper;

    @Resource
    private ProductInfoMapper productInfoMapper;

    @Override
    public int delByPrimaryKey(String guid) {
        return productTypeMapper.deleteByPrimaryKey(guid);
    }

    @Override
    public int save(ProductType record) {
        return productTypeMapper.insert(record);
    }

    @Override
    public int saveSelective(ProductType record) {
        return productTypeMapper.insertSelective(record);
    }

    @Override
    public ProductType queryByPrimaryKey(String guid) {
        return productTypeMapper.selectByPrimaryKey(guid);
    }

    @Override
    public int modifyByPrimaryKeySelective(ProductType record) {
        return productTypeMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public int modifyByPrimaryKeyWithBLOBs(ProductType record) {
        return productTypeMapper.updateByPrimaryKeyWithBLOBs(record);
    }

    @Override
    public int modifyByPrimaryKey(ProductType record) {
        return productTypeMapper.updateByPrimaryKey(record);
    }

    /**
     * 添加产品类别
     *
     * @param productTypeForm 前端参数
     * @return
     * @author 朱修贤
     */
    @Override
    @Transactional
    public int add(ProductTypeForm productTypeForm) {
        int result = 0;
        if (Objects.nonNull(productTypeForm)) {
            ProductType productType = BeanCopyUtil.convertBean(productTypeForm, ProductType.class);
            String parentId = productType.getParentId();
            if (StringUtils.isEmpty(parentId)) {
                productType.setParentId("#");
            }
            String guid = UUIDUtils.randomUUID();
            productType.setGuid(guid);
            productType.setTypeVal(getMaxCode());
            productType.setModifiedTime(new Date());
            productType.setState(1);
            multiAddExpansion(productTypeForm, guid);
            result = productTypeMapper.insertSelective(productType);
            if (result < 1) {
                throw new RuntimeException("新增产品类别失败");
            }
        }
        return result;
    }

    /**
     * 修改产品类别
     *
     * @param productTypeForm 前端参数
     * @return
     * @author 朱修贤
     */
    @Override
    @Transactional
    public int modify(ProductTypeForm productTypeForm) {
        int result = 0;
        if (Objects.nonNull(productTypeForm)) {
            ProductType productType = BeanCopyUtil.convertBean(productTypeForm, ProductType.class);
            String guid = productType.getGuid();
            if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
                productType.setModifiedTime(new Date());
                // 先删除数据库中产品类别对应的所有扩展信息,再重新添加前端传过来的扩展信息
                this.productTypeExpansionMapper.deleteByProductTypeGuid(guid);
                multiAddExpansion(productTypeForm, guid);
                productType.setModifiedTime(new Date());
                result = this.productTypeMapper.updateByPrimaryKeySelective(productType);
                if (result < 1) throw new RuntimeException("删除产品类别失败");
            }
        }
        return result;
    }

    /**
     * 添加多条产品类别扩展信息的方法(消除重复代码)
     *
     * @param productTypeForm 前端参数
     * @param guid            产品类别guid
     * @author 朱修贤
     */
    private void multiAddExpansion(ProductTypeForm productTypeForm, String guid) {
        int result;
        List<ProductTypeExpansionForm> productTypeExpansionForms = productTypeForm.getProductTypeExpansionForms();
        List<ProductTypeExpansion> productTypeExpansions = new ArrayList<>();
        if (Objects.nonNull(productTypeExpansionForms) && productTypeExpansionForms.size() > 0) {
            for (ProductTypeExpansionForm productTypeExpansionForm : productTypeExpansionForms) {
                ProductTypeExpansion productTypeExpansion = BeanCopyUtil.convertBean(productTypeExpansionForm, ProductTypeExpansion.class);
                productTypeExpansion.setGuid(UUIDUtils.randomUUID());
                productTypeExpansion.setCreateTime(new Date());
                productTypeExpansion.setFkProductTypeGuid(guid);
                productTypeExpansion.setModifiedTime(new Date());
                productTypeExpansion.setState(1);
                productTypeExpansions.add(productTypeExpansion);
            }
            result = this.productTypeExpansionMapper.batchInsert(productTypeExpansions);
            if (result < 1) {
                throw new RuntimeException("添加产品类别失败,批量添加产品类别扩展信息失败");
            }
        }
    }

    /**
     * 删除产品类别
     *
     * @param productTypeForm 前端参数
     * @return
     * @author 朱修贤
     */
    @Override
    @Transactional
    public int remove(ProductTypeForm productTypeForm) {
        int result = 0;
        if (Objects.nonNull(productTypeForm)) {
            ProductType productType = BeanCopyUtil.convertBean(productTypeForm, ProductType.class);
            String typeVal = productType.getTypeVal();
            // 查找该产品类型是否还存在已上架的商品
            if (StringUtils.isNotEmpty(typeVal) && typeVal.matches("^[T][0-9]{3}$")) {
                int goodsCount = this.goodsInfoMapper.selectGoodsCountByProductTypeValue(typeVal);
                if (goodsCount >= 1) { // 如果依然存在已上架的商品,则不能删除产品类型
                    throw new RuntimeException("该产品类别存在已上架的商品,不能删除该类别");
                } else {
                    String guid = productType.getGuid();
                    if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
                        // 删除产品类别扩展信息
                        result = this.productTypeExpansionMapper.deleteByProductTypeGuid(guid);
                        if (result < 0) throw new RuntimeException("删除产品类别失败 -- 删除产品扩展信息失败");

                        // 软删除对应的产品
                        result = this.productInfoMapper.softRemoveByProductTypeGuid(guid);
                        if (result < 0) throw new RuntimeException("软删除产品信息失败");

                        // 删除产品类别
                        result = this.productTypeMapper.deleteByPrimaryKey(guid);
                        if (result < 0) throw new RuntimeException("删除产品类别失败");
                    }
                }
            }

        }
        return result;
    }

    /**
     * 产品类别 -- 分页查询
     *
     * @param productTypeForm 前端参数
     * @return
     * @author 朱修贤
     */
    @Override
    public PageInfo<Page<Map>> queryListByPage(ProductTypeForm productTypeForm) {
        PageHelper.startPage(productTypeForm.getPageNo(), productTypeForm.getPageSize());
        Page page = this.productTypeMapper.selectListByPage(productTypeForm);
        if (Objects.nonNull(page) && page.size() > 0) {
            return new PageInfo(page);
        }
        return null;
    }

    /**
     * 查询所有产品类型,不分页
     *
     * @param productTypeForm
     * @return
     */
    @Override
    public List<ProductType> queryAll(ProductTypeForm productTypeForm) {
        log.info("ProductTypeForm ============> {}", productTypeForm);
        if (Objects.nonNull(productTypeForm)) {
            ProductType productType = BeanCopyUtil.convertBean(productTypeForm, ProductType.class);
            List<ProductType> productTypes = this.productTypeMapper.selectAll(productType);
            return productTypes;
        }
        return null;
    }

    @Override
    public int checkTypeVal(String typeVal) {
        int count = this.productTypeMapper.checkTypeVal(typeVal);
        return count;
    }

    private Integer getCurrentCode() {
        StringBuffer sbb = new StringBuffer();
        String maxCode = this.productTypeMapper.getCurrentCode();
        if (maxCode == null) {
            return 0;
        }
        sbb.append(CODE_PREFIX);
        String currentNum = maxCode.substring(sbb.toString().length());
        return Integer.valueOf(currentNum);
    }

    private String getMaxCode() {
        Integer currentNum = getCurrentCode();
        String maxCode = BusinessCodeRule.notYearCodeRule(CODE_PREFIX, currentNum, 4);
        return maxCode;
    }
}
