package com.hcloud.microserver.commoncore.interceptor;

import com.hcloud.microserver.commoncore.annotation.AuthOrgValidate;
import com.hcloud.microserver.commoncore.annotation.AuthValidate;
import com.hcloud.microserver.commoncore.base.BaseInterceptor;
import com.hcloud.microserver.commoncore.constant.GlobleConstant;
import com.hcloud.microserver.commoncore.enums.AuthEnum;
import com.hcloud.microserver.commoncore.enums.NoAuthEnum;
import com.hcloud.microserver.commoncore.enums.OrgAuthEnum;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.commoncore.util.TokenUtil;
import io.jsonwebtoken.Claims;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.method.HandlerMethod;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * 权限拦截器定义
 *
 * @author yys
 */
@Slf4j
@Configuration
public class AuthenticationAdvancedInterceptor extends BaseInterceptor {

    private static final String PERMISSION_KEY = "url";

    private static final String PERMISSION_IS_CHECKED = "isChecked";

    @Autowired
    private RedisTemplate redisTemplate;

    @SuppressWarnings("rawtypes")
    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws
            Exception {
        // 如果不是映射到方法直接通过
        if (!(handler instanceof HandlerMethod)) {
            return true;
        }
        HandlerMethod handlerMethod = (HandlerMethod) handler;
        Method method = handlerMethod.getMethod();
        String methodName = method.getName();
        //如果是不需要验证的method直接通过
        if (NoAuthEnum.isNoAuth(methodName)) {
            return true;
        }
        AuthOrgValidate authValidate = method.getAnnotation(AuthOrgValidate.class);
        //获取AccessToken值
        String accessToken = getAccessToken(request);
        //获取鉴权结果
        ResultEnum authResult = authUserPermission(accessToken, redisTemplate, authValidate);
        int resultCode = authResult.getCode();
        if (ResultEnum.SUCCESS.getCode() == resultCode) {
            return super.preHandle(request, response, handler);
        } else {
            responseMsg(authResult, response);
            return Boolean.FALSE;
        }
    }

    @Override
    public ResultEnum authUserPermission(String accessToken, RedisTemplate redisTemplate, Annotation authValidate) {
        AuthOrgValidate orgValidate = null;
        if (authValidate != null) {
            orgValidate = (AuthOrgValidate) authValidate;
        }
        if (StringUtils.isEmpty(accessToken) || GlobleConstant.FALSE.equals(accessToken)) {
            return checkAuthInfoNoAccessToken(orgValidate);
        } else {
            return checkAuthInfoByAccessToken(accessToken, orgValidate);
        }
    }

    /**
     * 无accessToken的处理方式
     *
     * @param orgValidate
     * @return
     */
    private ResultEnum checkAuthInfoNoAccessToken(AuthOrgValidate orgValidate) {
        if (orgValidate == null) {
            log.info("内部接口不能访问");
            return ResultEnum.PERMISSION_DENIED;
        }
        if (orgValidate.value().getAuthCode().equals(OrgAuthEnum.DEFAULT.getAuthCode())) {
            log.info("默认公共接口，无需授权访问");
            return ResultEnum.SUCCESS;
        } else {
            log.info("需要登录受权的接口");
            return ResultEnum.NOT_LOGIN;
        }
    }

    /**
     * 有accessToken的处理方式
     *
     * @param accessToken
     * @param orgValidate
     * @return
     */
    private ResultEnum checkAuthInfoByAccessToken(String accessToken, AuthOrgValidate orgValidate) {
        try {
            String authCode = orgValidate.value().getAuthCode();
            String authTarget = orgValidate.value().getAuthTarget();
            log.info("accessToken---------------" + accessToken);
            //从缓存取出登录后的信息
            Map loginMap = (Map) redisTemplate.opsForValue().get("carbon2:token:"+accessToken);
            if (loginMap == null || loginMap.size() == 0) {
                return ResultEnum.TOKEN_EXPIRED;
            } else {
                redisTemplate.opsForValue().set("carbon2:token:"+accessToken, loginMap, 2, TimeUnit.HOURS);
            }
            Claims claims = TokenUtil.parseJWT(accessToken);
            String subject = claims.getSubject();
            if (GlobleConstant.CURRENT_USER.equals(subject)) {
                if (orgValidate == null) {
                    log.info("内部访问接口，不对外提供服务");
                    return ResultEnum.PERMISSION_DENIED;
                } else if (authCode.equals(AuthEnum.DEFAULT.getAuthCode())
                        || authCode.equals(AuthEnum.COMMON_PRO_SERVICE.getAuthCode())) {
                    log.info("默认后台管理公共权限,不需要权限验证");
                    return ResultEnum.SUCCESS;
                } else {
                    log.info("后台管理权限,需要权限验证");
                    //用户权限列表
                    List<Map<String, Object>> userPermissionList =
                            (List<Map<String, Object>>) loginMap.get(GlobleConstant.AUTH_MAP);
                    if (null != userPermissionList && userPermissionList.size() > 0) {
                        boolean isAuth = false;
                        for (Map<String, Object> map : userPermissionList) {
                            String authStr = (String) map.get(PERMISSION_KEY);
                            Integer isChecked = (Integer) map.get(PERMISSION_IS_CHECKED);
                            log.info("authStr====================>{}", authStr);
                            if (authStr.equalsIgnoreCase(authTarget) && isChecked == 1) {
                                isAuth = true;
                                break;
                            }
                        }
                        if (isAuth) {
                            log.info("命中权限,可以直接访问");
                            return ResultEnum.SUCCESS;
                        } else {
                            return ResultEnum.PERMISSION_DENIED;
                        }
                    } else {
                        log.info("登录过期，请重新登录");
                        return ResultEnum.TOKEN_EXPIRED;
                    }
                }
            }
        } catch (Exception e) {
            log.info("accessToken解密错误，请查看解密信息");
            e.printStackTrace();
            return ResultEnum.PERMISSION_DENIED;
        }
        return ResultEnum.PERMISSION_DENIED;
    }
}