package com.hcloud.microserver.commoncore.interceptor;

import com.hcloud.microserver.commoncore.annotation.AuthOrgValidate;
import com.hcloud.microserver.commoncore.annotation.AuthValidate;
import com.hcloud.microserver.commoncore.base.BaseInterceptor;
import com.hcloud.microserver.commoncore.constant.GlobleConstant;
import com.hcloud.microserver.commoncore.enums.AuthEnum;
import com.hcloud.microserver.commoncore.enums.NoAuthEnum;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.commoncore.util.TokenUtil;
import io.jsonwebtoken.Claims;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.method.HandlerMethod;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.util.List;
import java.util.Map;

/**
 * 权限拦截器定义
 *
 * @author xiezt
 */
@Slf4j
@Configuration
public class AuthenticationInterceptor extends BaseInterceptor {

    private static final String PERMISSION_KEY = "url";

    @Autowired
    private RedisTemplate redisTemplate;

    @SuppressWarnings("rawtypes")
    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws
            Exception {
        // 如果不是映射到方法直接通过
        if (!(handler instanceof HandlerMethod)) {
            return true;
        }
        HandlerMethod handlerMethod = (HandlerMethod) handler;
        Method method = handlerMethod.getMethod();
        //如果是不需要验证的method直接通过
        if (NoAuthEnum.isNoAuth(handlerMethod.getMethod().getName())) {
            return true;
        }
        AuthValidate authValidate = method.getAnnotation(AuthValidate.class);
        //获取AccessToken值
        String accessToken = getAccessToken(request);
        //获取鉴权结果
        ResultEnum authResult = authUserPermission(accessToken, redisTemplate, authValidate);
        int resultCode = authResult.getCode();
        if (ResultEnum.SUCCESS.getCode() == resultCode) {
            return super.preHandle(request, response, handler);
        } else {
            responseMsg(authResult, response);
            return Boolean.FALSE;
        }
    }


    @Override
    public ResultEnum authUserPermission(String accessToken, RedisTemplate redisTemplate,
                                         Annotation authValidate) {
        String authCode = null;
        String authTarget = null;
        AuthValidate orgValidate = null;
        if (authValidate != null) {
            orgValidate = (AuthValidate) authValidate;
            authCode = orgValidate.value().getAuthCode();
            authTarget = orgValidate.value().getAuthTarget();
        }
        if (StringUtils.isEmpty(accessToken) || GlobleConstant.FALSE.equals(accessToken)) {
            if (authValidate == null) {
                log.info("内部接口不能访问");
                return ResultEnum.PERMISSION_DENIED;
            }
            if (orgValidate.value().getAuthCode().equals(AuthEnum.DEFAULT.getAuthCode())) {
                log.info("默认公共接口，无需授权访问");
                return ResultEnum.SUCCESS;
            } else {
                log.info("需要登录受权的接口");
                return ResultEnum.NOT_LOGIN;
            }
        } else {
            Claims claims;
            try {
                log.info("---------------" + accessToken);
                claims = TokenUtil.parseJWT(accessToken);
                String subject = claims.getSubject();
                if (GlobleConstant.CURRENT_MEMBER.equals(subject)) {
                    log.info("移动终端已登录用户，操作步骤");
                    if (authValidate == null) {
                        log.info("内部访问接口，不对外提供服务");
                        return ResultEnum.PERMISSION_DENIED;
                    } else if (authCode.equals(AuthEnum.COMMON_PRO_SERVICE.getAuthCode())) {
                        return ResultEnum.SUCCESS;
                    } else {
                        log.info("移动终端用户未授权访问");
                        return ResultEnum.FAILED;
                    }
                } else if (GlobleConstant.CURRENT_USER.equals(subject)) {
                    log.info("管理后台已登录用户，操作步骤");
                    if (authValidate == null) {
                        log.info("内部访问接口，不对外提供服务");
                        return ResultEnum.PERMISSION_DENIED;
                    } else if (authCode.equals(AuthEnum.DEFAULT.getAuthCode())
                            || authCode.equals(AuthEnum.COMMON_PRO_SERVICE.getAuthCode())) {
                        log.info("默认后台管理公共权限,不需要权限验证");
                        return ResultEnum.SUCCESS;
                    } else {
                        log.info("后台管理权限,需要权限验证");
                        //从缓存取出登录后的信息
                        Map loginMap = (Map) redisTemplate.opsForValue().get(accessToken);
                        //用户权限列表
                        if (loginMap == null || loginMap.size() == 0) {
                            return ResultEnum.TOKEN_EXPIRED;
                        }
                        List<Map<String, Object>> userPermissionList = (List<Map<String, Object>>) loginMap
                                .get(GlobleConstant.AUTH_LIST);
                        if (null != userPermissionList && userPermissionList.size() > 0) {
                            boolean is_auth = false;
                            for (Map<String, Object> map : userPermissionList) {
                                String authStr = (String) map.get(PERMISSION_KEY);
                                if (authStr.equalsIgnoreCase(authTarget)) {
                                    is_auth = true;
                                    break;
                                }
                            }
                            if (is_auth) {
                                log.info("命中权限,可以直接访问");
                                return ResultEnum.SUCCESS;
                            } else {
                                return ResultEnum.PERMISSION_DENIED;
                            }
                        }
                    }
                }
            } catch (Exception e) {
                log.info("accessToken解密错误，请查看解密信息");
                e.printStackTrace();
                return ResultEnum.PERMISSION_DENIED;
            }
        }
        return ResultEnum.FAILED;
    }
}
