package com.hcloud.microserver.commoncore.interceptor;

import static com.hcloud.microserver.commoncore.enums.AuthEnum.NO_AUTH;
import static com.hcloud.microserver.commoncore.enums.AuthEnum.SYS_LOGIN;

import com.hcloud.microserver.commoncore.annotation.AuthValidate;
import com.hcloud.microserver.commoncore.base.BaseInterceptor;
import com.hcloud.microserver.commoncore.constant.GlobleConstant;
import com.hcloud.microserver.commoncore.enums.NoAuthEnum;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.commoncore.service.RedisUtils;
import com.hcloud.microserver.commoncore.util.TokenUtil;
import io.jsonwebtoken.Claims;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.util.Map;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.util.CollectionUtils;
import org.springframework.web.method.HandlerMethod;

/**
 * 权限拦截器定义
 *
 * @author xiezt
 */
@Slf4j
@Configuration
public class SupplyChainAuthenticationInterceptor extends BaseInterceptor {

    private static final String PERMISSION_KEY = "url";

    @Autowired
    private RedisTemplate redisTemplate;

    @Autowired
    private RedisUtils redisUtils;

    @SuppressWarnings("rawtypes")
    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response,
                             Object handler)
            throws Exception {

        String origin = request.getHeader("Origin");
        response.setHeader("Access-Control-Allow-Origin", origin);
        response.setHeader("Access-Control-Allow-Methods", "*");
        response.setHeader("Access-Control-Allow-Headers",
                "Origin,Content-Type,Accept,token,X-Requested-With");
        response.setHeader("Access-Control-Allow-Credentials", "true");

        // 如果不是映射到方法直接通过
        if (!(handler instanceof HandlerMethod)) {
            return true;
        }
        HandlerMethod handlerMethod = (HandlerMethod) handler;
        Method method = handlerMethod.getMethod();
        //如果是不需要验证的method直接通过
        if (NoAuthEnum.isNoAuth(handlerMethod.getMethod().getName())) {
            return true;
        }
        if ("error".equals(method.getName())) {
            return true;
        }
        AuthValidate authValidate = method.getAnnotation(AuthValidate.class);
        //获取AccessToken值
        String accessToken = getAccessTokenFromHeader(request);
        //获取鉴权结果
        ResultEnum authResult = authUserPermission(accessToken, redisTemplate, authValidate);
        int resultCode = authResult.getCode();
        if (ResultEnum.SUCCESS.getCode() == resultCode) {
            return super.preHandle(request, response, handler);
        } else {
            responseMsg(authResult, response);
            return Boolean.FALSE;
        }
    }

    @Override
    public ResultEnum authUserPermission(String accessToken, RedisTemplate redisTemplate,
                                         Annotation authValidate) {
        String authCode = null;
        String authTarget = null;
        AuthValidate normalValidate = null;
        if (authValidate == null) {
            log.info("内部访问接口，不对外提供服务");
            return ResultEnum.PERMISSION_DENIED;
        } else {
            normalValidate = (AuthValidate) authValidate;
            authCode = normalValidate.value().getAuthCode();
            authTarget = normalValidate.value().getAuthTarget();
        }
        if (authCode.equals(NO_AUTH.getAuthCode()) || authCode.equals(SYS_LOGIN.getAuthCode())) {
            log.info("开放访问登录接口权限");
            return ResultEnum.SUCCESS;
        } else if (StringUtils.isEmpty(accessToken) || GlobleConstant.FALSE.equals(accessToken)) {
            return ResultEnum.NOT_LOGIN;
        } else {
            Claims claims;
            try {

                log.info("---------------" + accessToken);
                String token = accessToken.substring(0, accessToken.length() - 32);
                String code = accessToken
                        .substring(accessToken.length() - 32, accessToken.length());

                String userId = TokenUtil.getUserId(accessToken);
                Map cacheMap = (Map) redisUtils.getValue(userId);
                if (CollectionUtils.isEmpty(cacheMap)) {
                    return ResultEnum.TOKEN_EXPIRED;
                }
                String onlyCode = (String) cacheMap.get("onlyCode");
                String relToken = (String) cacheMap.get("accessToken");
                String subject = TokenUtil.parseJWT(token).getSubject();
                String relSubject = TokenUtil.parseJWT(relToken).getSubject();

                if (!code.equals(onlyCode) && subject.equals(relSubject)) {
                    log.info("该账户已在其它地方登陆");
                    return ResultEnum.ACCOUNT_ALREADY_LOGIN;
                } else {
                    return ResultEnum.SUCCESS;
                }
            } catch (Exception e) {
                log.info("accessToken解密错误，请查看解密信息");
                e.printStackTrace();
                return ResultEnum.PERMISSION_DENIED;
            }
        }
    }
}
